package se.chalmers.cs.gf.oaa;

import se.gu.ling.trindikit.oaa.common.*;
import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.*;
import se.chalmers.cs.gf.translate.*;
import se.chalmers.cs.gf.util.Pair;

import java.util.*;

/**
 *  Implements the translation solvable.
 */
public class Translate extends OAASolver {

        private static String solvableString = "translate(Grammar,FromLang,Input,ToLang,Output)";

        private Translators translators;

        public Translate(Translators translators) {
                super(solvableString);
                this.translators = translators;
        }

        public boolean solve(IclTerm goal, IclList params, IclList answers) {
                String grammar;
                IclTerm fromLangTerm;
                String toLang;
                String input;
                IclTerm result;

                try {
                        grammar = ((IclStr)goal.getTerm(0)).toUnquotedString();
                        fromLangTerm = goal.getTerm(1);
                        input    = ((IclStr)goal.getTerm(2)).toUnquotedString();
                        toLang   = ((IclStr)goal.getTerm(3)).toUnquotedString();
                        result   = goal.getTerm(4);
                } catch (ClassCastException e){
                        return false;
                }

                Translator translator = translators.getTranslator(grammar);
                if (translator == null) {
                        System.err.println("Unknown grammar " + grammar);
                        return false;
                }

                Collection<Pair<String,Tree>> ps 
                        = Parse.parse(translator, fromLangTerm, input);
                
                boolean success = false;
                for (Pair<String,Tree> p : ps) {
                        try {
                                String fromLang = p.fst;
                                Tree tree = p.snd;
                                String resultStr = translator.linearize(toLang, tree);
                                IclTerm linResult = new IclStr(resultStr);
                                IclTerm r = Unifier.getInstance().unify(result, linResult);
                                if (r != null) {
                                        IclTerm ans = new IclStruct("translate", 
                                                                    new IclStr(grammar),
                                                                    new IclStr(fromLang), 
                                                                    new IclStr(input), 
                                                                    new IclStr(toLang), 
                                                                    r);
                                        System.err.println("Adding answer " + ans);
                                        answers.add(ans);
                                        success = true;   
                                }
                        } catch (GFException ex) {
                                ex.printStackTrace();
                        }
                }

                System.err.println("Returning " + success);
                return success;
        }
    
}
