package se.chalmers.cs.gf.parse.chart;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import static se.chalmers.cs.gf.parse.ParseLogger.*;

/**
 *  A simple lexer.
 */
public class Lexer {

        private Pattern tokenPatt;

        public Lexer() {
                // simple words are any non-empty sequence of 
                // lower case letters, upper case letters, 
                // title case letters, decimal digits and
                // connector punctuation.
                String word = "[\\p{Ll}\\p{Lu}\\p{Lt}\\p{Lo}\\p{Nd}\\p{Pc}]+";
                // quoted words are strings of any character non double quote
                // enclosed in double quotes
                // FIXME: allow escaped double quotes and backslashes
                String quoted = "\"(\\\\\"|[^\"])*\"";
                // punctuation is a non-empty string of punctuation characters
                String punct = "[\\.\\,\\?\\!\\(\\)\\:\\;\\-\\[\\]\\{\\}&\\$]+";
                tokenPatt = Pattern.compile("(" + word 
                                            + "|" + quoted 
                                            + "|" + punct 
                                            + ")");
        }

        // FIXME: discards input that it doesn't understand
        public List<Token> lex(CharSequence input) {
                List<Token> ts = new ArrayList<Token>();

                Matcher m = tokenPatt.matcher(input);
                while (m.find()) {
                        String s = m.group();
                        ts.add(new Token(s));
                }

                log.finest("Lexed \"" + input + "\" to " + ts);

                return ts;
        }

        public static void main(String[] args) throws java.io.IOException {
                Lexer l = new Lexer();
                BufferedReader in 
                        = new BufferedReader(new InputStreamReader(System.in));
                String line;
                while ((line = in.readLine()) != null)
                        System.out.println(l.lex(line));
        }

}
