package se.chalmers.cs.gf.parse.chart;

/**
 *  A parse chart edge produced by a non-terminal rule.
 */
public class NonTerminalEdge extends Edge {

        private NonTerminalRule rule;

        /** Position of the dot in the rule. */
        private int pos;

        private NonTerminalEdge(NonTerminalRule rule, int pos) {
                this.rule = rule;
                this.pos = pos;
        }

        public NonTerminalEdge(NonTerminalRule rule) {
                this(rule, 0);
        }

        public NonTerminalEdge(NonTerminalEdge e) {
                this(e.rule, e.pos);
        }

        /**
         *  Get the rule that this edge is for.
         */
        public NonTerminalRule getRule() {
                return rule;
        }

        /**
         *  Checks if the next symbol needed is the given category.
         */
        public boolean needs(Category c) {
                return isActive() && ((NonTerminalRule)rule).getRhs().get(pos).equals(c);
        }

        /**
         *  Checks if this edge is active, that is, if it is still missing 
         *  some symbol.
         */
        public boolean isActive() {
                return pos < rule.getSize();
        }

        /**
         *  Move the dot one step forward.
         */
        public void stepForward() {
                if (!isActive())
                        throw new IllegalStateException(
                                "Can't go forward in a passive rule.");
                pos++;
        }
        
        public boolean equals(Object o) {
                return o instanceof NonTerminalEdge && equals((NonTerminalEdge)o);
        }

        public boolean equals(NonTerminalEdge e) {
                return rule.equals(e.rule) && pos == e.pos;
        }

        public int hashCode() {
                return rule.hashCode() + pos;
        }

        public String toString() {
                StringBuilder sb = new StringBuilder();
                sb.append(rule);
                sb.append(" (").append(pos).append(")");
                return sb.toString();
        }

}
